;; Generated package description from flymake-codespell.el  -*- no-byte-compile: t -*-
(define-package "flymake-codespell" "0.1.0.20231030.222337" "Flymake backend for codespell" '((emacs "26.1") (compat "29.1.4.2")) :commit "d72e3ad4cdfd5cea1509a241d55e18f54ae2aeca" :authors '(("Stefan Kangas" . "stefankangas@gmail.com")) :maintainer '("Stefan Kangas" . "stefankangas@gmail.com") :keywords '("extensions") :url "https://www.github.com/skangas/flymake-codespell")
