;;; Directory Local Variables         -*- no-byte-compile: t; -*-
;;; For more information see (info "(emacs) Directory Variables")

((nil . ((sentence-end-double-space . t)
         (emacs-lisp-docstring-fill-column . 65)
         (fill-column . 70)))
 (emacs-lisp-mode . ((indent-tabs-mode . nil)))
 (lisp-data-mode . ((indent-tabs-mode . nil))))
